@@ -3,14 +3,13 @@
 __all__ = ["Annotation", "AnnotationResult"]
 
 from abc import ABC, abstractmethod
-from collections import OrderedDict
-from typing import Any, Dict, List, Optional, Tuple
+from dataclasses import dataclass
+from typing import Any, Dict, List, Optional
 
 from ..execution import is_complexity_tracing_enabled, MemoryFootprint
 
 
 _TRACKED_ANNOTATIONS = []
-_GROUP_INDICES = {}
 _ANNOTATION_COUNTER = 0
 
 
@@ -71,23 +70,12 @@ class Annotation(ABC):
     
     def _track(self) -> None:
         """
-        Tracks this annotation in ``_TRACKED_ANNOTATIONS`` and updates ``_GROUP_INDICES`` with the
-        index of the annotation if ``self.group`` is present. If the annotation has children
+        Tracks this annotation in ``_TRACKED_ANNOTATIONS``. If the annotation has children
         (returned by ``self.children``), the children are removed from ``_TRACKED_ANNOTATIONS``.
         """
         if is_complexity_tracing_enabled():
             return
 
-        global _GROUP_INDICES, _TRACKED_ANNOTATIONS
-
-        idx = len(_TRACKED_ANNOTATIONS)
-        if self.name not in _GROUP_INDICES:
-            _GROUP_INDICES[self.name] = []
-        if self.limit is not None and len(_GROUP_INDICES[self.name]) >= self.limit:
-            return
-        else:
-            _GROUP_INDICES[self.name].append(idx)
-    
         for child in self.children:
             try:
                 _TRACKED_ANNOTATIONS.remove(child)
@@ -112,9 +100,8 @@ class Annotation(ABC):
         Resets the list of tracked annotations and the mapping of group names to indices in that
         list.
         """
-        global _ANNOTATION_COUNTER, _GROUP_INDICES, _TRACKED_ANNOTATIONS
+        global _ANNOTATION_COUNTER, _TRACKED_ANNOTATIONS
         _TRACKED_ANNOTATIONS.clear()
-        _GROUP_INDICES.clear()
         _ANNOTATION_COUNTER = 0
 
     @property
@@ -270,6 +257,26 @@ class Annotation(ABC):
         }
 
 
+@dataclass
+class AnnotationResultMessage:
+    """
+    A data class representing a success or failure message generated by an annotation.
+
+    Args:
+        message (``Optional[str]``): the message text
+        name (``Optional[str]``): the annotation name
+        satisfied (``bool``): whether the annotation providing the message was satisfied
+    """
+    message: Optional[str]
+    """the message text"""
+
+    name: Optional[str]
+    """the annotation name"""
+
+    satisfied: bool
+    """whether the annotation was satisfied"""
+
+
 class AnnotationResult:
     """
     Class that manages and defines an API for interacting with the results of an annotation.
@@ -369,21 +376,20 @@ class AnnotationResult:
         return self._value
 
     @property
-    def messages(self) -> List[Tuple[str, Optional[str], bool]]: # (message, name, satisfied)
+    def messages(self) -> List[AnnotationResultMessage]:
         """
-        ``list[tuple[str, Optional[str], bool]]``: The messages returned by this annotation and its 
-        children based on whether or not the annotations were satisfied. A list of tuples where the 
-        first element is the message, the second is the name of the annotation (or ``None`` if no 
-        name is  present), and the third is whether the annotation was satisfied.
+        ``list[AnnotationResultMessage]``: the messages returned by this annotation and its 
+        children based on whether or not the annotations were satisfied
         """
         messages = []
         for c in self.children:
             messages.extend(c.messages)
-        
-        if self.satisfied and self.annotation.success_message:
-            messages.append((self.annotation.success_message, self.annotation.name, True))
-        elif not self.satisfied and self.annotation.failure_message:
-            messages.append((self.annotation.failure_message, self.annotation.name, False))
+
+        messages.append(AnnotationResultMessage(
+            self.annotation.success_message if self.satisfied else self.annotation.failure_message,
+            self.annotation.name,
+            self.satisfied,
+        ))
 
         return messages
 
